//********************************************************************
//*** ResourcePool/oracle/Transaction.t.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: Transaction.t.cc,v 1.17 2005/01/06 16:28:00 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "ResourcePool/oracle/ResourcePool.hh"
#include "t/main.hh"

using namespace std;
using namespace fatalmind;

typedef ResourcePool<ResourceType::oracle> RP;

#include "testconfig.cc"


std::auto_ptr<RP> _rp;

void test_init()
{
	if (_rp.get() == 0) {
		ResourcePoolOptions opt;
		opt.setMaxExecTry(1);
		_rp.reset(new RP(getFactory(), opt));
	}
}

void INIT_oracle_Transaction() 
{
	test_init();
	init(*_rp);	
	RP::Command::SQLExecute init("insert into hirsch (id, ukn) values (?, ?)");
	for (int i = 0; i < 300; ++i) {
		init.bind(0, i);
		init.bind<std::string>(1, "hirschig");
		_rp->execute(init);
	}
}

void oracle_init_me(RP& p) {
	RP::Command::SQLExecute init("update hirsch set ukn=?");
	init.bind<std::string>(0, "hohoho ' kasdfjs \" sdf");
	p.execute(init);
}

void ora_trans_test_ok(RP& p, int id, bool cache = false) {
	oracleHandlesWithCache<DefaultThreadedModel >& h = _rp->get();
	RP::Command::Transaction t;
	
	RP::Command::SQLExecute cmd("update hirsch set ukn=? where id =?");
	cmd.bind(1, id*2);
	cmd.bind<std::string>(0, "hiiiirsch'");
	t.add(cmd);
	cmd.bind(1, id*2+1);
	t.add(cmd);

	p.execute(t);

	RP::Command::SQLSelectRow sel("select ukn from hirsch where id = ?");
	sel.bind(0, id*2);
	std::string ukn;
	sel.bindout(0, ukn);
	sel.execute(h);
	assertEqual(ukn, "hiiiirsch'");
	_rp->free(h);
}

void TEST_oracle_Transaction_one()
{
	oracle_init_me(*_rp);
	ora_trans_test_ok(*_rp, 0);
}

void TEST_PERF_oracle_Transaction_one()
{
	oracle_init_me(*_rp);
	for (int i = 0; i < 100; ++i) {
		ora_trans_test_ok(*_rp, i);
	}
}

void TEST_PERF_oracle_Transaction_one_cached()
{
	oracle_init_me(*_rp);
	for (int i = 0; i < 100; ++i) {
		ora_trans_test_ok(*_rp, i, true);
	}
}

void ora_trans_test_rec_ok(RP& p, int id, bool fail = false) {
	RP::Command::Transaction t;
	RP::Command::Transaction t2;
	
	RP::Command::SQLExecute cmd("update hirsch set ukn=? where id =?");
	cmd.bind(1, id*3);
	cmd.bind<std::string>(0, "hiiiirsch'");
	t.add(cmd);
	cmd.bind(1, id*3+1);
	t.add(cmd);

	cmd.bind(fail?2:1, id*3+2);
	t2.add(cmd);
	t2.add(t);

	try {
		p.execute(t2);
		if (fail) {
			assertNotReach();	
		}
	} catch (SQL::SQLException) {
		if (!fail) {
			assertNotReach();	
		}
	}
}

void TEST_oracle_Transaction_rec()
{
	oracleHandlesWithCache<DefaultThreadedModel >& h = _rp->get();
	oracle_init_me(*_rp);
	ora_trans_test_rec_ok(*_rp, 0);

	RP::Command::SQLSelectRow sel("select ukn from hirsch where id = ?");
	sel.bind(0, 0);
	std::string ukn;
	sel.bindout(0, ukn);
	sel.execute(h);
	assertEqual(ukn, "hiiiirsch'");
}

void TEST_oracle_Transaction_rec_fail()
{
	oracleHandlesWithCache<DefaultThreadedModel >& h = _rp->get();
	oracle_init_me(*_rp);
	ora_trans_test_rec_ok(*_rp, 0, true);

	RP::Command::SQLSelectRow sel("select ukn from hirsch where id = ?");
	sel.bind(0, 1);
	std::string ukn;
	sel.bindout(0, ukn);
	sel.execute(h);
	assertEqual(ukn, "hohoho ' kasdfjs \" sdf");
}

void ora_trans_test_fail(RP& p, int id, bool cached = false) {
	RP::Command::Transaction t;
	
	RP::Command::SQLExecute cmd("update hirsch set ukn=? where id =?");
	cmd.bind<std::string>(0, "gnu");
	cmd.bind(1, id*2);
	t.add(cmd);

	std::string chk;
	RP::Command::SQLSelectRow chksel("select ukn from hirsch where id = ?");
	chksel.bind(0, id*2);
	chksel.bindout(0, chk);
	t.add(chksel);

	RP::Command::SQLExecute cmd2("update hirsch set ukn=? where id =?");
	cmd2.bind(1, id*2+1);
	t.add(cmd2);
	

	try {
		p.execute(t);
		assertNotReach();
	} catch (OracleException e) {
	}
	RP::Command::SQLSelectRow sel("select ukn from hirsch where id = ?");
	sel.bind(0, id*2);
	std::string ukn;
	sel.bindout(0, ukn);
	p.execute(sel);
	assertEqual(ukn, "hohoho ' kasdfjs \" sdf");
	assertEqual(chk, "gnu");
}

void TEST_oracle_Transaction_fail()
{
	oracle_init_me(*_rp);
	ora_trans_test_fail(*_rp, 0);
}

void TEST_PERF_oracle_Transaction_fail()
{
	oracle_init_me(*_rp);
	for (int i = 0; i < 100; ++i) {
		ora_trans_test_fail(*_rp, i);
	}
}

void TEST_PERF_oracle_Transaction_fail_cached()
{
	oracle_init_me(*_rp);
	for (int i = 0; i < 100; ++i) {
		ora_trans_test_fail(*_rp, i, true);
	}
}


void ora_trans_test_fail_rec(RP& p, int id, bool cache = false) {
	RP::Command::Transaction t;
	RP::Command::Transaction subt;
	
	RP::Command::SQLExecute cmd("update hirsch set ukn=? where id =?");
	cmd.bind<std::string>(0, "gnu2");
	cmd.bind(1, id*2);
	subt.add(cmd);
	t.add(subt);

	std::string chk;
	RP::Command::SQLSelectRow chksel("select ukn from hirsch where id = ?");
	chksel.bind(0, id*2);
	chksel.bindout(0, chk);
	t.add(chksel);

	RP::Command::SQLExecute cmd2("update hirsch set ukn=? where id =?");
	cmd2.bind(1, id*2+1);
	t.add(cmd2);
	
	try {
		p.execute(t);
		assertNotReach();
	} catch (OracleException e) {
	}
	RP::Command::SQLSelectRow sel("select ukn from hirsch where id = ?");
	sel.bind(0, id*2);
	std::string ukn;
	sel.bindout(0, ukn);
	p.execute(sel);
	assertEqual(ukn, "hohoho ' kasdfjs \" sdf");
	assertEqual(chk, "gnu2");
}

void TEST_oracle_Transaction_fail_rec() {
	oracle_init_me(*_rp);
	ora_trans_test_fail_rec(*_rp, 0);
}

void TEST_PERF_oracle_Transaction_fail_rec() {
	oracle_init_me(*_rp);
	for (int i = 0; i < 100; ++i) {
		ora_trans_test_fail_rec(*_rp, i);
	}
}

void TEST_PERF_oracle_Transaction_fail_rec_ca() {
	oracle_init_me(*_rp);
	for (int i = 0; i < 100; ++i) {
		ora_trans_test_fail_rec(*_rp, i, true);
	}
}
